
#include <Arduino_JSON.h>
#include <ESP8266WiFi.h>
#include <MQTT.h>
#include <ESP8266WebServer.h>

#define LED_PIN LED_BUILTIN //LED_BUILTIN is built in LED

String inputString = "";         // a String to hold incoming data
bool stringComplete = false;  // whether the string is complete

WiFiClient net;
MQTTClient MQTTclient; 
ESP8266WebServer server(80);

//WiFi
const char ssid[] = "????";
const char pass[] = "???";
IPAddress local_IP(192, 168, 1, 17);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0); 

// the IP address for the MQTT server
char MQTTip[] = "192.168.1.71";   

unsigned long UpdateCount = 6;
unsigned long TickCount = UpdateCount-1;
unsigned long PacketCount = 0;
unsigned long lastMillis = 0;

void handleRoot() {

  //http://192.168.1.17/?fan=1&speed=120
  //http://192.168.1.17/?relay=1
  /*
  String message = "RESPONSE\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  */
  
  String message = "";
  
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + "=" + server.arg(i) + "\n";
  }

  String payload = server.argName(0) + " " + server.arg(0);

  Serial.println(payload);


  inputString = "";
  
  int tout = 1000;
  
  while( (!Serial.available()) && ((--tout) > 0) )
  {  
    delay(1);
    
    if(tout <= 0)
    {
      message = "IO TIMEOUT";
      break;
    }
    else//GET RESPONSE
    {
      while (Serial.available()) 
      {    
        char inChar = (char)Serial.read();
        
        if( (inChar == '\r') || (inChar == '\n') )
        { 
          message += inputString;      
        }
        else
        {
          inputString += inChar;
        }
        
      }//End of while (Serial.available()) 

      break;
    
    }//End of if(tout <= 0)
    
  }//End of while( (!Serial.available()) && ((--tout) > 0) )
  
  //message += " TOUT=" + String(tout);  
     
  server.send(200, "text/plain", message);
  
}


void handleNotFound(){
  
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
 
}

void setup() {

  pinMode(LED_PIN, OUTPUT);
    
  Serial.begin(115200);

 // Configures static IP address
  if (!WiFi.config(local_IP, gateway, subnet)) {
    Serial.println("STA Failed to configure");
  }
  
  WiFi.begin(ssid, pass);

  // Note: Local domain names (e.g. "Computer.local" on OSX) are not supported by Arduino.
  // You need to set the IP address directly.
  MQTTclient.begin(MQTTip, net);
  MQTTclient.onMessage(messageReceived);

  WiFiconnect();
  
  MQTTclient.publish("/ervcontrol/status", "start");    

  MQTTclient.publish("/ervcontrol/status", "run");
  
  server.on("/", handleRoot);

  server.on("/status", [](){
    
    server.send(200, "text/plain", "machine status");
  });

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("/ervcontrol/status HTTP server started");
  MQTTclient.publish("/ervcontrol/status", "HTTP server started");
}

void loop() {
  
  MQTTclient.loop();
  
  delay(10);  // <- fixes some issues with WiFi stability

  if (!MQTTclient.connected()) {
    WiFiconnect();
  }

  server.handleClient();
    
  // publish a message roughly every second.
  if (millis() - lastMillis > (UpdateCount * 1000)) 
  { 
     lastMillis = millis();
     SendMQTT();
  }
  
  delay(100);
}


void WiFiconnect() {
  
  Serial.print("checking wifi...");
  Serial.println(ssid);
  
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
  }
  
  Serial.println("\nWiFi Connected! ");
  Serial.println( WiFi.localIP());

  Serial.print("\nMQTT connecting...");
  while (!MQTTclient.connect(WiFi.localIP().toString().c_str() , "try", "try")) {
    Serial.print(".");
    delay(800);
    digitalWrite(LED_PIN, LOW);
    delay(200);
    digitalWrite(LED_PIN, HIGH);
  }

  Serial.println("\nMQTT Connected!");

  digitalWrite(LED_PIN, LOW);
  
  MQTTclient.subscribe("/ervcontrol/control");  
  MQTTclient.publish("/ervcontrol/status", "start");
  MQTTclient.publish("/ervcontrol/localip", "/ervcontrol/localip/" + WiFi.localIP().toString());

}

void SendMQTT()
{
  digitalWrite(LED_PIN, LOW);
    
  JSONVar myArray;

  
  
  //STALE
  myArray[0] = analogRead(0);
  myArray[1] = analogRead(1);
  myArray[2] = analogRead(2);
  myArray[3] = analogRead(3);
  myArray[4] = analogRead(4);
  myArray[5] = analogRead(5);

  
  //MISC
  myArray[6] = (PacketCount++);
  myArray[7] = WiFi.RSSI();
  
   
  String jsonString = JSON.stringify(myArray);
  
  MQTTclient.publish("/ervcontrol/sensors", jsonString);
  
  digitalWrite(LED_PIN, HIGH);
}

//Test MQTT mon
//Topic: /ervcontrol/control
//Payload: RELAYS 1 0 1 0 1 1 

void messageReceived(String &topic, String &payload) {

  digitalWrite(LED_PIN, LOW);
  
  if(topic == "/ervcontrol/control")
  {  
    if(payload.startsWith("read"))
    {
       SendMQTT();
       MQTTclient.publish("/ervcontrol/localip", "/ervcontrol/localip/" + WiFi.localIP().toString());
    }
    else
    {
      Serial.println(payload);
    }
  
    MQTTclient.publish("/ervcontrol/payload", topic + "/" + payload);

  }

  digitalWrite(LED_PIN, HIGH);
 
}
